unit FormDatenEinlesen;

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, DB,
  SQLite3Conn, SQLDB, IDataExchange;

type
  { TFormDatenEinlesen }
  TFormDatenEinlesen = class(TForm)
    BtnOK: TButton;
    BtnCancel: TButton;
    BtnSelectFile: TButton;
    EditTrennzeichen: TEdit;
    EditName: TEdit;
    EditComment: TMemo;
    Label1: TLabel;
    LabelAnzZahl: TLabel;
    LabelAnzWrt: TLabel;
    LabelGrTXT: TLabel;
    LabelAnzZeich: TLabel;
    OpenDialog: TOpenDialog;
    Panel1: TPanel;
    StaticText1: TStaticText;
    StaticText2: TStaticText;
    procedure BtnCancelClick(Sender: TObject);
    procedure BtnOKClick(Sender: TObject);
    procedure BtnSelectFileClick(Sender: TObject);
    function DetectMostCommonDelimiter(FileContent: TStringList; IncludeLineBreak: Boolean): Char;
    function ShowDelimiterChoiceDialog(SuggestedDelimiter: Char): Integer;
    procedure CountWordsAndNumbers(FileContent: TStringList; Delimiter: Char; out TotalWords, TotalNumbers: Integer);
  private
    SelectedFilePath: string;
    FSQLite3Connection: TSQLite3Connection;
    FSQLTransaction: TSQLTransaction;
    FQuery: TSQLQuery;
    FDataExchange: IDataExch;
  public
    property SQLite3Connection: TSQLite3Connection read FSQLite3Connection write FSQLite3Connection;
    property SQLTransaction: TSQLTransaction read FSQLTransaction write FSQLTransaction;
    property DataExchange: IDataExch read FDataExchange write FDataExchange;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

var
  frmDatenEinlesen: TFormDatenEinlesen;

implementation

{$R *.lfm}

{ TFormDatenEinlesen }

constructor TFormDatenEinlesen.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  // Initialize FQuery with nil and later link with SQLite3Connection and SQLTransaction
  FQuery := TSQLQuery.Create(nil);
end;

destructor TFormDatenEinlesen.Destroy;
begin
  FQuery.Free;
  inherited Destroy;
end;



procedure TFormDatenEinlesen.BtnSelectFileClick(Sender: TObject);
var
  FileContent: TStringList;
  SuggestedDelimiter: Char;
  UserChoice: Integer;
  CustomDelimiter: string;
  TotalChars, TotalWords, TotalNumbers: Integer;
  FileStream: TFileStream;
begin
  // Sicherstellen, dass das OpenDialog-Objekt existiert
  if not Assigned(OpenDialog) then
    OpenDialog := TOpenDialog.Create(Self);

  if OpenDialog.Execute then
  begin
    SelectedFilePath := OpenDialog.FileName;
    if FileExists(SelectedFilePath) then  // Überprüfen Sie, ob die Datei existiert
    begin
      try
        FileStream := TFileStream.Create(SelectedFilePath, fmOpenRead or fmShareDenyNone);
        try
          LabelGrTXT.Caption := Format('Größe der Textdatei: %d Bytes', [FileStream.Size]);
        finally
          FileStream.Free;
        end;
      except
        on E: Exception do
        begin
          ShowMessage('Fehler beim Öffnen der Datei: ' + E.Message);
          Exit;
        end;
      end;

      FileContent := TStringList.Create;
      try
        FileContent.LoadFromFile(SelectedFilePath);
        TotalChars := Length(FileContent.Text);
        LabelAnzZeich.Caption := Format('Anzahl Zeichen: %d', [TotalChars]);

        SuggestedDelimiter := DetectMostCommonDelimiter(FileContent, True);

        UserChoice := ShowDelimiterChoiceDialog(SuggestedDelimiter);
        case UserChoice of
          1: if SuggestedDelimiter = #10 then
               EditTrennzeichen.Text := 'Umbruch'
             else
               EditTrennzeichen.Text := SuggestedDelimiter;
          2: begin
               CustomDelimiter := InputBox('Eigener Trennzeichen', 'Geben Sie Ihr Trennzeichen ein:', '');
               if Length(CustomDelimiter) > 0 then
                 EditTrennzeichen.Text := CustomDelimiter[1]
               else
                 EditTrennzeichen.Text := '';
             end;
          3: EditTrennzeichen.Text := '';
        end;

        if EditTrennzeichen.Text <> '' then
        begin
          if EditTrennzeichen.Text = 'Umbruch' then
            CountWordsAndNumbers(FileContent, #10, TotalWords, TotalNumbers)
          else if Length(EditTrennzeichen.Text) > 0 then
            CountWordsAndNumbers(FileContent, EditTrennzeichen.Text[1], TotalWords, TotalNumbers);

          LabelAnzWrt.Caption := Format('Anzahl Wörter: %d', [TotalWords]);
          LabelAnzZahl.Caption := Format('Anzahl Zahlen: %d', [TotalNumbers]);
        end;
      finally
        FileContent.Free;
      end;
    end
    else
    begin
      ShowMessage('Die ausgewählte Datei existiert nicht.');
    end;
  end;
end;


function TFormDatenEinlesen.DetectMostCommonDelimiter(FileContent: TStringList; IncludeLineBreak: Boolean): Char;
var
  DelimiterCounts: array[Char] of Integer;
  Line: string;
  Delimiters: string;
  I, MaxCount: Integer;
  CurrentChar: Char;
begin
  for CurrentChar := Low(DelimiterCounts) to High(DelimiterCounts) do
    DelimiterCounts[CurrentChar] := 0;

  Delimiters := ',; ';

  for Line in FileContent do
  begin
    for I := 1 to Length(Line) do
    begin
      CurrentChar := Line[I];
      if CurrentChar in [',', ';', ' '] then
        Inc(DelimiterCounts[CurrentChar]);
    end;
  end;

  if IncludeLineBreak then
    Inc(DelimiterCounts[#10], FileContent.Count - 1);

  MaxCount := 0;
  Result := ',';
  for CurrentChar := Low(DelimiterCounts) to High(DelimiterCounts) do
  begin
    if DelimiterCounts[CurrentChar] > MaxCount then
    begin
      MaxCount := DelimiterCounts[CurrentChar];
      Result := CurrentChar;
    end;
  end;
end;

function TFormDatenEinlesen.ShowDelimiterChoiceDialog(SuggestedDelimiter: Char): Integer;
var
  DelimiterChar: Char;
begin
  if SuggestedDelimiter = #10 then
    DelimiterChar := 'U'
  else
    DelimiterChar := SuggestedDelimiter;

  Result := MessageDlg(Format('Das vorgeschlagene Trennzeichen ist "%s". Wählen Sie eine Option:',
                              [DelimiterChar]),
                       mtConfirmation,
                       [mbYes, mbNo, mbCancel], 0);
end;

procedure TFormDatenEinlesen.CountWordsAndNumbers(FileContent: TStringList; Delimiter: Char; out TotalWords, TotalNumbers: Integer);
var
  Line, Word: string;
  I, Value: Integer;
  Words: TStringList;
begin
  TotalWords := 0;
  TotalNumbers := 0;
  Words := TStringList.Create;
  try
    Words.Delimiter := Delimiter;
    Words.StrictDelimiter := True;
    for Line in FileContent do
    begin
      if Delimiter = #10 then
        Words.DelimitedText := Line
      else
        Words.DelimitedText := StringReplace(Line, Delimiter, #10, [rfReplaceAll]);

      for Word in Words do
      begin
        Inc(TotalWords);
        if TryStrToInt(Word, Value) then
          Inc(TotalNumbers);
      end;
    end;
  finally
    Words.Free;
  end;
end;

procedure TFormDatenEinlesen.BtnOKClick(Sender: TObject);
var
  CurrentDate: TDateTime;
  DataSetID: Integer;
begin
  if SelectedFilePath <> '' then
  begin
    CurrentDate := Now;  // Gets the current date and time
    if FSQLTransaction.Active then
      FSQLTransaction.Rollback;

    FSQLite3Connection.StartTransaction;
    try
      FQuery.DataBase := FSQLite3Connection;
      FQuery.Transaction := FSQLTransaction;
      FQuery.SQL.Text := 'INSERT INTO DataSet (Name, Comment, CreationDate) VALUES (:Name, :Comment, :CreationDate);';
      FQuery.ParamByName('Name').AsString := EditName.Text;
      FQuery.ParamByName('Comment').AsString := EditComment.Text;
      FQuery.ParamByName('CreationDate').AsDateTime := CurrentDate;  // Correctly pass the current date and time
      FQuery.ExecSQL;

      FQuery.SQL.Text := 'SELECT last_insert_rowid() AS DataSetID;';
      FQuery.Open;
      DataSetID := FQuery.FieldByName('DataSetID').AsInteger;
      FQuery.Close;

      FQuery.SQL.Text := 'INSERT INTO UnsortedValues (DataSetID, Value) VALUES (:DataSetID, :Value);';
      FQuery.ParamByName('DataSetID').AsInteger := DataSetID;
      FQuery.ParamByName('Value').LoadFromFile(SelectedFilePath, ftBlob);
      FQuery.ExecSQL;

      FSQLTransaction.Commit;

      Self.Close;

      if Assigned(FDataExchange) then
        FDataExchange.UpdateAndSelect(DataSetID);
    except
      on E: Exception do
      begin
        FSQLTransaction.Rollback;
        ShowMessage('Error saving: ' + E.Message);
      end;
    end;
  end
  else
    ShowMessage('No file selected.');
end;

procedure TFormDatenEinlesen.BtnCancelClick(Sender: TObject);
begin
  Close;
end;

end.

